/*
 *                           TERMS AND CONDITIONS
 *                                   FOR
 *                         OPEN SOURCE CODE LICENSE
 *                               Version 1.1
 * 
 * Japan Registry Services Co., Ltd. ("JPRS"), a Japanese corporation
 * having its head office at Chiyoda First Bldg. East 13F 3-8-1 Nishi-Kanda,
 * Chiyoda-ku, Tokyo 101-0065, Japan, grants you the license for open source
 * code specified in EXHIBIT A the "Code" subject to the following Terms and
 * Conditions ("OSCL").
 * 
 * 1. License Grant.
 *   JPRS hereby grants you a worldwide, royalty-free, non-exclusive
 *   license, subject to third party intellectual property claims:
 *   (a) under intellectual property rights (other than patent or
 *       trademark) licensable by JPRS to use, reproduce, modify, display,
 *       perform, sublicense and distribute the Code (or portions thereof)
 *       with or without modifications, and/or as part of a derivative work;
 *       or
 *   (b) under claims of the infringement through the making, using,
 *       offering to sell and/or otherwise disposing the JPRS Revised Code
 *       (or portions thereof);
 *   (c) the licenses granted in this Section 1(a) and (b) are effective on
 *       the date JPRS first distributes the Code to you under the terms of
 *       this OSCL;
 *   (d) Notwithstanding the above stated terms, no patent license is
 *       granted:
 *       1)  for a code that you delete from the Code;
 *       2)  separate from the Code; or
 *       3)  for infringements caused by:
 *            i) modification of the Code; or
 *           ii) combination of the Code with other software or devices.
 * 
 * 2. Consents.
 *   You agree that:
 *   (a) you must include a copy of this OSCL and the notice set forth in
 *       EXHIBIT A with every copy of the Code you distribute;
 *   (b) you must include a copy of this OSCL and the notice set forth in
 *       EXHIBIT A with every copy of binary form of the Code in the
 *       documentation and/or other materials provided with the distribution;
 *   (c) you may not offer or impose any terms on any source code version
 *       that alters or restricts the applicable version of this OSCL or
 *       the recipients' rights hereunder.
 *   (d) If the terms and conditions are set forth in EXHIBIT A, you must
 *       comply with those terms and conditions.
 * 
 * 3. Proprietary Information.
 *   All trademarks, service marks, patents, copyrights, trade secrets, and
 *   other proprietary rights in or related to the Code are and will remain
 *   the exclusive property of JPRS or its licensors, whether or not
 *   specifically recognized or perfected under local law except specified
 *   in this OSCL; provided however you agree and understand that the JPRS
 *   name may not be used to endorse or promote this Code without prior
 *   written approval of JPRS.
 * 
 * 4. WARRANTY DISCLAIMER.
 *   JPRS MAKES NO REPRESENTATIONS AND WARRANTIES REGARDING THE USE OF THE
 *   CODE, NOR DOES JPRS MAKE ANY REPRESENTATIONS THAT THE CODE WILL BECOME
 *   COMMERCIALLY AVAILABLE. JPRS, ITS AFFILIATES, AND ITS SUPPLIERS DO NOT
 *   WARRANT OR REPRESENT THAT THE CODE IS FREE OF ERRORS OR THAT THE CODE
 *   IS SUITABLE FOR TRANSLATION AND/OR LOCALIZATION. THE CODE IS PROVIDED
 *   ON AN "AS IS" BASIS AND JPRS AND ITS SUPPLIERS HAVE NO OBLIGATION TO
 *   CORRECT ERRORS OR TO SUPPORT THE CODE UNDER THIS OSCL FOR ANY REASON.
 *   TO THE FULL EXTENT PERMITTED BY LAW, ALL OBLIGATIONS ARE HEREBY
 *   EXCLUDED WHETHER EXPRESS, STATUTORY OR IMPLIED UNDER LAW, COURSE OF
 *   DEALING, CUSTOM, TRADE USAGE, ORAL OR WRITTEN STATEMENT OR OTHERWISE,
 *   INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY
 *   OR FITNESS FOR A PARTICULAR PURPOSE CONCERNING THE CODE.
 * 
 * 5. NO LIABILITY.
 *   UNDER NO CIRCUMSTANCES SHALL JPRS AND/OR ITS AFFILIATES, LICENSORS, OR
 *   REPRESENTATIVES BE LIABLE FOR ANY DAMAGES INCLUDING BUT NOT LIMITED TO
 *   CONSEQUENTIAL, INDIRECT, SPECIAL, PUNITIVE OR INCIDENTAL DAMAGES,
 *   WHETHER FORESEEABLE OR UNFORESEEABLE, BASED ON YOUR CLAIMS, INCLUDING,
 *   BUT NOT LIMITED TO, CLAIMS FOR LOSS OF DATA, GOODWILL, PROFITS, USE OF
 *   MONEY, INTERRUPTION IN USE OR AVAILABILITY OF DATA, STOPPAGE, IMPLIED
 *   WARRANTY, BREACH OF CONTRACT, MISREPRESENTATION, NEGLIGENCE, STRICT
 *   LIABILITY IN TORT, OR OTHERWISE.
 * 
 * 6. Indemnification.
 *   You hereby agree to indemnify, defend, and hold harmless JPRS for any
 *   liability incurred by JRPS due to your terms of warranty, support,
 *   indemnity, or liability offered by you to any third party.
 * 
 * 7. Termination.
 * 7.1 This OSCL shall be automatically terminated in the events that:
 *   (a) You fail to comply with the terms herein and fail to cure such
 *       breach within 30 days of becoming aware of the breach;
 *   (b) You initiate patent or copyright infringement litigation against
 *       any party (including a cross-claim or counterclaim in a lawsuit)
 *       alleging that the Code constitutes a direct or indirect patent or
 *       copyright infringement, in such case, this OSCL to you shall
 *       terminate as of the date such litigation is filed;
 * 7.2 In the event of termination under Sections 7.1(a) or 7.1(b) above,
 *     all end user license agreements (excluding distributors and
 *     resellers) which have been validly granted by You or any distributor
 *     hereunder prior to termination shall survive termination.
 *
 * 
 * 8. General.
 *   This OSCL shall be governed by, and construed and enforced in
 *   accordance with, the laws of Japan. Any litigation or arbitration
 *   between the parties shall be conducted exclusively in Tokyo, Japan
 *   except written consent of JPRS provides other venue.
 * 
 * 
 *                                EXHIBIT A
 * 
 * The original open source code of idnkit-2 is idnkit-1.0 developed and
 * conceived by Japan Network Information Center ("JPNIC"), a Japanese
 * association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
 * Chiyoda-ku, Tokyo 101-0047, Japan, and JPRS modifies above original code
 * under following Terms and Conditions set forth by JPNIC.
 * 
 *                                  JPNIC
 * 
 * Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved.
 * 
 * By using this file, you agree to the terms and conditions set forth bellow.
 * 
 *                       LICENSE TERMS AND CONDITIONS
 * 
 * The following License Terms and Conditions apply, unless a different
 * license is obtained from Japan Network Information Center ("JPNIC"),
 * a Japanese association, Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda,
 * Chiyoda-ku, Tokyo 101-0047, Japan.
 * 
 * 1. Use, Modification and Redistribution (including distribution of any
 *    modified or derived work) in source and/or binary forms is permitted
 *    under this License Terms and Conditions.
 * 
 * 2. Redistribution of source code must retain the copyright notices as they
 *    appear in each source code file, this License Terms and Conditions.
 * 
 * 3. Redistribution in binary form must reproduce the Copyright Notice,
 *    this License Terms and Conditions, in the documentation and/or other
 *    materials provided with the distribution. For the purposes of binary
 *    distribution the "Copyright Notice" refers to the following language:
 *    "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
 * 
 * 4. The name of JPNIC may not be used to endorse or promote products
 *    derived from this Software without specific prior written approval of
 *    JPNIC.
 * 
 * 5. Disclaimer/Limitation of Liability: THIS SOFTWARE IS PROVIDED BY JPNIC
 *    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *    PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL JPNIC BE LIABLE
 *    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *    BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *    WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *    ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * 
 *                        JPRS Public License Notice
 *                                   For
 *                                idnkit-2.
 * 
 * The contents of this file are subject to the Terms and Conditions for
 * the Open Source Code License (the "OSCL"). You may not use this file
 * except in compliance with above terms and conditions. A copy of the OSCL
 * is available at <http://jprs.co.jp/idn/>.
 * The JPRS Revised Code is idnkit-2.
 * The Initial Developer of the JPRS Revised Code is Japan Network
 * Information Center ("JPNIC"), a Japanese association,
 * Kokusai-Kougyou-Kanda Bldg 6F, 2-3-4 Uchi-Kanda, Chiyoda-ku, Tokyo
 * 101-0047, Japan.
 * "Copyright (c) 2000-2002 Japan Network Information Center.  All rights reserved."
 * "Copyright (c) 2010-2012 Japan Registry Services Co., Ltd.  All rights reserved."
 * Contributor(s): ______________________________________.
 * 
 * If you wish to allow use of your version of this file only under the
 * above License(s) and not to allow others to use your version of this
 * file, please indicate your decision by deleting the relevant provisions
 * above and replacing them with the notice and other provisions required
 * by the above License(s). If you do not delete the relevant provisions,
 * a recipient may use your version of this file under either the above
 * License(s).
 */

#include <Python.h>
#include <structmember.h>
#include <idn/res.h>
#include <idn/version.h>

#include "idnkitIDNA.h"
#include "idnkitIDNA2008.h"
#include "idnkitError.h"

/*
 * 'IDNA2008' object.
 */
typedef struct {
    PyObject_HEAD
    idn_resconf_t ctx;
} IDNA2008;

#define INITIAL_STRING_BUFFER_SIZE	256

static long
adjust_action_value(long actions);
static PyObject *
IDNA2008_new(PyTypeObject *type, PyObject *args, PyObject *keywords);
static void
IDNA2008_dealloc(PyObject *self);
static int
IDNA2008_sysinit(PyObject *type, PyObject *args, PyObject *keywords);
static PyObject *
IDNA2008_init(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_encode(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_decode(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_compare(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_check(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_set_local_check_file(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_get_unicode_version(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_get_idna_table_version(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_get_libidn_version(PyObject *self, PyObject *args);
static PyObject *
IDNA2008_get_version(PyObject *self, PyObject *args);

/*
 * Adjust action value (IDNA_* to IDN_*).
 */
static long
adjust_action_value(long actions) {
    if (actions == IDNA_ENCODE_REGIST)
	return IDN_ENCODE_REGIST | IDN_LOCALCHECK;
    if (actions == IDNA_ENCODE_LOOKUP)
	return IDN_ENCODE_LOOKUP | IDN_LOCALCHECK;
    if (actions == IDNA_DECODE_REGIST)
	return IDN_DECODE_REGIST | IDN_LOCALCHECK;
    if (actions == IDNA_DECODE_LOOKUP)
	return IDN_DECODE_LOOKUP | IDN_LOCALCHECK;
    if (actions == IDNA_COMPARE_REGIST)
	return IDN_COMPARE_REGIST | IDN_LOCALCHECK;
    if (actions == IDNA_COMPARE_LOOKUP)
	return IDN_COMPARE_LOOKUP | IDN_LOCALCHECK;
    if (actions == IDNA_CHECK_REGIST)
	return IDN_CHECK_REGIST | IDN_LOCALCHECK;
    if (actions == IDNA_CHECK_LOOKUP)
	return IDN_CHECK_LOOKUP | IDN_LOCALCHECK;
    
    return actions;
}

/*
 * __new__() method.
 */
static PyObject *
IDNA2008_new(PyTypeObject *type, PyObject *args, PyObject *keywords) {
    IDNA2008 *self;
    idn_resconf_t ctx;

    if (idn_resconf_create(&ctx) != idn_success)
	return NULL;

    self = (IDNA2008 *)type->tp_alloc(type, 0);
    if (self == NULL) {
	idn_resconf_destroy(ctx);
	return NULL;
    }
    self->ctx = ctx;

    return (PyObject *)self;
}

/*
 * __del__() method.
 */
static void
IDNA2008_dealloc(PyObject *self) {
    idn_resconf_destroy(((IDNA2008 *)self)->ctx);
    self->ob_type->tp_free(self);
}

/*
 * __init__() method.
 */
static int
IDNA2008_sysinit(PyObject *type, PyObject *args, PyObject *keywords) {
    return 0;
}

/*
 * init() method.
 */
static PyObject *
IDNA2008_init(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    char *filename = NULL;

    if (!PyArg_ParseTuple(args, "|s", &filename))
	return NULL;

    if (filename == NULL)
	r = idn_resconf_setdefaults(((IDNA2008 *)self)->ctx);
    else if (*filename == '\0')
	r = idn_resconf_loadfile(((IDNA2008 *)self)->ctx, NULL);
    else
	r = idn_resconf_loadfile(((IDNA2008 *)self)->ctx, filename);

    if (r != idn_success) {
	Error_set_object(r);
	return NULL;
    }

    return Py_BuildValue("i", 1);
}

/*
 * encode() method.
 */
static PyObject *
IDNA2008_encode(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    long actions = IDNA_ENCODE_REGIST;
    char *from = NULL;
    char *to = NULL;
    char *totmp = NULL;
    size_t tolen = INITIAL_STRING_BUFFER_SIZE;
    PyObject *obj;

    if (!PyArg_ParseTuple(args, "s|l", &from, &actions))
	return NULL;

    for (;;) {
	totmp = (char *) realloc(to, tolen);
	if (totmp == NULL) {
	    free(to);
	    return Py_BuildValue("is", idn_nomemory, NULL);
	}
	to = totmp;
	r = idn_res_encodename(((IDNA2008 *)self)->ctx,
	    adjust_action_value(actions), from, to, tolen);
	if (r == idn_success)
	    break;
	else if (r != idn_buffer_overflow) {
	    free(to);
	    Error_set_object(r);
	    return NULL;
	}
	tolen <<= 1;
    }

    obj = Py_BuildValue("s", to);
    free(to);
    return obj;
}

/*
 * decode() method.
 */
static PyObject *
IDNA2008_decode(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    long actions = IDNA_DECODE_REGIST;
    char *from = NULL;
    char *to = NULL;
    char *totmp = NULL;
    size_t tolen = INITIAL_STRING_BUFFER_SIZE;
    PyObject *obj;

    if (!PyArg_ParseTuple(args, "s|l", &from, &actions))
	return NULL;

    for (;;) {
	totmp = (char *) realloc(to, tolen);
	if (totmp == NULL) {
	    free(to);
	    return Py_BuildValue("is", idn_nomemory, NULL);
	}
	to = totmp;
	r = idn_res_decodename(((IDNA2008 *)self)->ctx, 
	    adjust_action_value(actions), from, to, tolen);
	if (r == idn_success)
	    break;
	else if (r != idn_buffer_overflow) {
	    free(to);
	    Error_set_object(r);
	    return NULL;
	}
	tolen <<= 1;
    }

    obj = Py_BuildValue("s", to);
    free(to);
    return obj;
}

/*
 * compare() method.
 */
static PyObject *
IDNA2008_compare(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    long actions1 = IDNA_COMPARE_LOOKUP;
    long actions2 = IDNA_COMPARE_LOOKUP;
    long tmp = 0;
    char *name1 = NULL;
    char *name2 = NULL;
    PyObject *obj;

    if (!PyArg_ParseTuple(args, "ss|ll", &name1, &name2, &actions1, &actions2))
	return NULL;
    if (!PyArg_ParseTuple(args, "ss|ll", &name1, &name2, &actions1, &tmp))
	return NULL;
    if (actions2 != tmp)
	actions2 = actions1;

    r = idn_res_comparenames2(((IDNA2008 *)self)->ctx, 
	adjust_action_value(actions1), name1, adjust_action_value(actions2),
	name2);
    if (r == idn_success)
	obj = Py_BuildValue("i", 1);
    else if (r == idn_neq)
	obj = Py_BuildValue("i", 0);
    else {
	Error_set_object(r);
	return NULL;
    }

    return obj;
}

/*
 * check() method.
 */
static PyObject *
IDNA2008_check(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    long actions = IDNA_CHECK_REGIST;
    char *name = NULL;
    PyObject *obj;

    if (!PyArg_ParseTuple(args, "s|l", &name, &actions))
	return NULL;

    r = idn_res_checkname(((IDNA2008 *)self)->ctx, 
	adjust_action_value(actions), name);

    if (r == idn_success) {
	obj = Py_BuildValue("i", 1);
    } else if (r == idn_prohcheck_error ||
	r == idn_unascheck_error ||
	r == idn_nfccheck_error ||
	r == idn_prefcheck_error ||
	r == idn_hyphcheck_error ||
	r == idn_combcheck_error ||
	r == idn_ctxjcheck_error ||
	r == idn_ctxocheck_error ||
	r == idn_bidicheck_error ||
	r == idn_localcheck_error ||
	r == idn_lencheck_error ||
	r == idn_rtcheck_error ||
	r == idn_tr46check_error) {
	obj = Py_BuildValue("i", 0);
    } else {
	Error_set_object(r);
	return NULL;
    }

    return obj;
}

/*
 * set_local_check_file() method.
 */
static PyObject *
IDNA2008_set_local_check_file(PyObject *self, PyObject *args) {
    idn_result_t r = idn_success;
    char *filename = NULL;

    if (!PyArg_ParseTuple(args, "|s", &filename))
	return NULL;

    r = idn_resconf_setlocalcheckfile(((IDNA2008 *)self)->ctx, filename);
    if (r != idn_success) {
	Error_set_object(r);
	return NULL;
    }

    return Py_BuildValue("i", 1);
}

/*
 * get_unicode_version() method.
 */
static PyObject *
IDNA2008_get_unicode_version(PyObject *self, PyObject *args) {
    return Py_BuildValue("s", idn_version_unicode());
}

/*
 * get_idna_table_version() method.
 */
static PyObject *
IDNA2008_get_idna_table_version(PyObject *self, PyObject *args) {
    return Py_BuildValue("s", idn_version_idnatable());
}

/*
 * get_libidn_version() method.
 */
static PyObject *
IDNA2008_get_libidn_version(PyObject *self, PyObject *args) {
    return Py_BuildValue("s", idn_version_libidn());
}

/*
 * get_version() method.
 */
static PyObject *
IDNA2008_get_version(PyObject *self, PyObject *args) {
    return Py_BuildValue("s", idn_version_getstring());
}

/*
 * List of members.
 */
static PyMemberDef IDNA2008_members[] = {
    {NULL}  /* Sentinel */
};

/*
 * List of methods.
 */
static PyMethodDef IDNA2008_methods[] = {
    {"encode", IDNA2008_encode, METH_VARARGS,
     "Encode a domain name.\n"},
    {"decode", IDNA2008_decode, METH_VARARGS,
     "Decode a domain name.\n"},
    {"compare", IDNA2008_compare, METH_VARARGS,
     "Compare two domain names.\n"},
    {"check", IDNA2008_check, METH_VARARGS,
     "Validate a domain name.\n"},
    {"get_unicode_version", IDNA2008_get_unicode_version, METH_NOARGS,
     "Get version of Unicode.\n"},
    {"get_idna_table_version", IDNA2008_get_idna_table_version, METH_NOARGS,
     "Get version of IDNA table.\n"},
    {"get_libidn_version", IDNA2008_get_libidn_version, METH_NOARGS,
     "Get version of libidnkitlite.\n"},
    {"get_version", IDNA2008_get_version, METH_NOARGS,
     "Get version information.\n"},
    {"init", IDNA2008_init, METH_VARARGS,
     "Initialize a conversion context.\n"},
    {"set_local_check_file", IDNA2008_set_local_check_file, METH_VARARGS,
     "Set code point set for local-check.\n"},
    {NULL}  /* Sentinel */
};

/*
 * Type object.
 */
static PyTypeObject IDNA2008_type = {
    PyObject_HEAD_INIT(NULL)
    0,                                        /* ob_size */
    "idnkit.IDNA2008",                        /* tp_name */
    sizeof(IDNA2008),                         /* tp_basicsize */
    0,                                        /* tp_itemsize */
    (destructor)IDNA2008_dealloc,             /* tp_dealloc */
    0,                                        /* tp_print */
    0,                                        /* tp_getattr */
    0,                                        /* tp_setattr */
    0,                                        /* tp_compare */
    0,                                        /* tp_repr */
    0,                                        /* tp_as_number */
    0,                                        /* tp_as_sequence */
    0,                                        /* tp_as_mapping */
    0,                                        /* tp_hash  */
    0,                                        /* tp_call */
    0,                                        /* tp_str */
    0,                                        /* tp_getattro */
    0,                                        /* tp_setattro */
    0,                                        /* tp_as_buffer */
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /* tp_flags */
    "IDNA2008 conversion context",            /* tp_doc */
    0,                                        /* tp_traverse */
    0,                                        /* tp_clear */
    0,                                        /* tp_richcompare */
    0,                                        /* tp_weaklistoffset */
    0,                                        /* tp_iter */
    0,                                        /* tp_iternext */
    IDNA2008_methods,                         /* tp_methods */
    IDNA2008_members,                         /* tp_members */
    0,                                        /* tp_getset */
    0,                                        /* tp_base */
    0,                                        /* tp_dict */
    0,                                        /* tp_descr_get */
    0,                                        /* tp_descr_set */
    0,                                        /* tp_dictoffset */
    (initproc)IDNA2008_sysinit,               /* tp_init */
    0,                                        /* tp_alloc */
    IDNA2008_new,                             /* tp_new */
};

/*
 * PyType_Ready.
 */
int
IDNA2008_type_ready(void) {
    if (idn_resconf_initialize() != idn_success)
	return -1;
    return PyType_Ready(&IDNA2008_type);
}

/*
 * PyModule_AddObject.
 */
void
IDNA2008_add_to_module(PyObject *module) {
    Py_INCREF(&IDNA2008_type);
    PyModule_AddObject(module, "IDNA2008", (PyObject *)&IDNA2008_type);

    PyModule_AddIntConstant(module, "IDNA2008_MAP",           IDN_MAP);
    PyModule_AddIntConstant(module, "IDNA2008_ASCLOWER",      IDN_ASCLOWER);
    PyModule_AddIntConstant(module, "IDNA2008_RTCONV",        IDN_RTCONV);
    PyModule_AddIntConstant(module, "IDNA2008_PROHCHECK",     IDN_PROHCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_UNASCHECK",     IDN_UNASCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_NFCCHECK",      IDN_NFCCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_PREFCHECK",     IDN_PREFCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_HYPHCHECK",     IDN_HYPHCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_COMBCHECK",     IDN_COMBCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_CTXJCHECK",     IDN_CTXJCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_CTXOCHECK",     IDN_CTXOCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_CTXOLITECHECK", IDN_CTXOLITECHECK);
    PyModule_AddIntConstant(module, "IDNA2008_BIDICHECK",     IDN_BIDICHECK);
    PyModule_AddIntConstant(module, "IDNA2008_LOCALCHECK",    IDN_LOCALCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_IDNCONV",       IDN_IDNCONV);
    PyModule_AddIntConstant(module, "IDNA2008_LENCHECK",      IDN_LENCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_RTCHECK",       IDN_RTCHECK);
    PyModule_AddIntConstant(module, "IDNA2008_UNDOIFERR",     IDN_UNDOIFERR);
}

/*
 * Create an 'IDNA2008' object.
 */
PyObject *
IDNA2008_create_object(void) {
    return IDNA2008_new(&IDNA2008_type, NULL, NULL);
}

